@@ -1,4 +1,4 @@
-# Copyright 2008-2023  Sutou Kouhei <kou@cozmixng.org>
+# Copyright (C) 2008-2025  Sutou Kouhei <kou@cozmixng.org>
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -14,16 +14,127 @@
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
-begin
-  require_relative "pkg-config/version"
-rescue LoadError
-end
-
-require "English"
 require "pathname"
 require "rbconfig"
 require "shellwords"
 
+module PKGConfig
+  VERSION = "1.6.2"
+
+  @@paths = []
+  @@override_variables = {}
+
+  module_function
+  def add_path(path)
+    @@paths << path
+  end
+
+  def set_override_variable(key, value)
+    @@override_variables[key] = value
+  end
+
+  def msvc?
+    /mswin/.match(RUBY_PLATFORM) and /^cl\b/.match(RbConfig::CONFIG["CC"])
+  end
+
+  def package_config(package)
+    PackageConfig.new(package,
+                      :msvc_syntax => msvc?,
+                      :override_variables => @@override_variables,
+                      :paths => @@paths)
+  end
+
+  def exist?(pkg)
+    package_config(pkg).exist?
+  end
+
+  def libs(pkg)
+    package_config(pkg).libs
+  end
+
+  def libs_only_l(pkg)
+    package_config(pkg).libs_only_l
+  end
+
+  def libs_only_L(pkg)
+    package_config(pkg).libs_only_L
+  end
+
+  def cflags(pkg)
+    package_config(pkg).cflags
+  end
+
+  def cflags_only_I(pkg)
+    package_config(pkg).cflags_only_I
+  end
+
+  def cflags_only_other(pkg)
+    package_config(pkg).cflags_only_other
+  end
+
+  def modversion(pkg)
+    package_config(pkg).version
+  end
+
+  def description(pkg)
+    package_config(pkg).description
+  end
+
+  def variable(pkg, name)
+    package_config(pkg).variable(name)
+  end
+
+  def check_version?(pkg, major=0, minor=0, micro=0)
+    return false unless exist?(pkg)
+    ver = modversion(pkg).split(".").collect {|item| item.to_i}
+    (0..2).each {|i| ver[i] = 0 unless ver[i]}
+
+    (ver[0] > major ||
+     (ver[0] == major && ver[1] > minor) ||
+     (ver[0] == major && ver[1] == minor &&
+      ver[2] >= micro))
+  end
+
+  def have_package(pkg, major=nil, minor=0, micro=0)
+    message = "#{pkg}"
+    unless major.nil?
+      message << " version (>= #{major}.#{minor}.#{micro})"
+    end
+    major ||= 0
+    result = checking_for(checking_message(message), "%s") do
+      if check_version?(pkg, major, minor, micro)
+        "yes (#{modversion(pkg)})"
+      else
+        if exist?(pkg)
+          "no (#{modversion(pkg)})"
+        else
+          "no (nonexistent)"
+        end
+      end
+    end
+    enough_version = result.start_with?("yes")
+    if enough_version
+      libraries = libs_only_l(pkg)
+      dldflags = libs(pkg)
+      dldflags = (Shellwords.shellwords(dldflags) -
+                  Shellwords.shellwords(libraries))
+      dldflags = dldflags.map {|s| /\s/ =~ s ? "\"#{s}\"" : s }.join(" ")
+      $libs   += " " + libraries
+      if /mswin/ =~ RUBY_PLATFORM
+        $DLDFLAGS += " " + dldflags
+      else
+        $LDFLAGS += " " + dldflags
+      end
+      $CFLAGS += " " + cflags_only_other(pkg)
+      if defined?($CXXFLAGS)
+        $CXXFLAGS += " " + cflags_only_other(pkg)
+      end
+      $INCFLAGS += " " + cflags_only_I(pkg)
+    end
+    enough_version
+  end
+end
+
 class PackageConfig
   class Error < StandardError
   end
@@ -229,21 +340,29 @@ class PackageConfig
         if brew
           homebrew_repository = run_command("brew", "--repository")
           if homebrew_repository
-          homebrew_repository_candidates <<
-            Pathname(homebrew_repository.to_s)
+            homebrew_repository_candidates <<
+              Pathname(homebrew_repository.strip)
           end
         end
         homebrew_repository_candidates.uniq.each do |candidate|
-          pkgconfig_base_path = candidate + "Library/Homebrew/os/mac/pkgconfig"
-          path = pkgconfig_base_path + mac_os_version
-          unless path.exist?
-            path = pkgconfig_base_path + mac_os_version.gsub(/\.\d+\z/, "")
+          mac_pkgconfig_base_path =
+            candidate + "Library/Homebrew/os/mac/pkgconfig"
+          mac_pkgconfig_path = mac_pkgconfig_base_path + mac_os_version
+          unless mac_pkgconfig_path.exist?
+            mac_pkgconfig_path =
+              mac_pkgconfig_base_path + mac_os_version.gsub(/\.\d+\z/, "")
           end
-          paths << path.to_s if path.exist?
+          paths << mac_pkgconfig_path.to_s if mac_pkgconfig_path.exist?
+
+          pkgconfig_path = candidate + "lib/pkgconfig"
+          paths << pkgconfig_path.to_s if pkgconfig_path.exist?
         end
       end
       paths.concat(default_paths)
-      paths.join(SEPARATOR)
+      [
+        with_config("pkg-config-path") || ENV["PKG_CONFIG_PATH"],
+        *paths,
+      ].compact.join(SEPARATOR)
     end
 
     def run_command(*command_line)
@@ -277,8 +396,10 @@ class PackageConfig
       @name = name
     end
     @options = options
-    path = @options[:path] || ENV["PKG_CONFIG_PATH"]
-    @paths = [path, self.class.default_path].compact.join(SEPARATOR).split(SEPARATOR)
+    @paths = [
+      @options[:path],
+      self.class.default_path,
+    ].compact.join(SEPARATOR).split(SEPARATOR)
     @paths.unshift(*(@options[:paths] || []))
     @paths = normalize_paths(@paths)
     @msvc_syntax = @options[:msvc_syntax]
@@ -289,6 +410,14 @@ class PackageConfig
     @override_variables = default_override_variables.merge(@override_variables)
   end
 
+  def eql?(other)
+    other.is_a?(self.class) and @name == other.name
+  end
+
+  def hash
+    @name.hash
+  end
+
   def exist?
     not pc_path.nil?
   end
@@ -303,7 +432,7 @@ class PackageConfig
 
   def cflags
     path_flags, other_flags = collect_cflags
-    (other_flags + path_flags).join(" ")
+    (path_flags + other_flags).join(" ")
   end
 
   def cflags_only_I
@@ -315,26 +444,25 @@ class PackageConfig
   end
 
   def libs
-    path_flags, other_flags = collect_libs
-    (path_flags + other_flags).join(" ")
+    collect_libs.join(" ")
   end
 
   def libs_only_l
-    collect_libs[1].find_all do |arg|
+    collect_libs.find_all do |arg|
       if @msvc_syntax
-        /\.lib\z/ =~ arg
+        arg.end_with?(".lib")
       else
-        /\A-l/ =~ arg
+        arg.start_with?("-l")
       end
     end.join(" ")
   end
 
   def libs_only_L
-    collect_libs[0].find_all do |arg|
+    collect_libs.find_all do |arg|
       if @msvc_syntax
-        /\A\/libpath:/ =~ arg
+        arg.start_with?("/libpath:")
       else
-        /\A-L/ =~ arg
+        arg.start_with?("-L")
       end
     end.join(" ")
   end
@@ -377,40 +505,32 @@ class PackageConfig
     @path_position
   end
 
-  def collect_requires(processed_packages={}, &block)
-    packages = []
-    targets = yield(self)
-    targets.each do |name|
-      next if processed_packages.key?(name)
-      package = self.class.new(name, @options)
-      processed_packages[name] = package
-      packages << package
-      packages.concat(package.collect_requires(processed_packages, &block))
-    end
-    packages_without_self = packages.reject do |package|
-      package.name == @name
-    end
-    packages_without_self.uniq do |package|
-      package.name
+  def collect_requires(&block)
+    dependencies = {}
+    pending_packages = yield(self).collect {|name| self.class.new(name, @options)}
+    until pending_packages.empty?
+      package = pending_packages.shift
+      next if dependencies.key?(package)
+      dependencies[package] = true
+      targets = yield(package)
+      targets.each do |name|
+        require_package = self.class.new(name, @options)
+        pending_packages.push(require_package)
+      end
     end
+    dependencies.keys
   end
 
   private
-  def sort_packages(packages)
-    packages.sort_by.with_index do |package, i|
-      [package.path_position, i]
-    end
-  end
-
   def collect_cflags
-    target_packages = sort_packages([self, *all_required_packages])
+    target_packages = [self, *all_required_packages]
     cflags_set = []
     target_packages.each do |package|
       cflags_set << package.declaration("Cflags")
     end
     all_cflags = normalize_cflags(Shellwords.split(cflags_set.join(" ")))
     path_flags, other_flags = all_cflags.partition {|flag| /\A-I/ =~ flag}
-    path_flags = normalize_path_flags(path_flags, "-I")
+    path_flags = path_flags.collect {|flag| normalize_path_flag(flag, "-I")}
     path_flags = path_flags.reject do |flag|
       flag == "-I/usr/include"
     end
@@ -423,20 +543,18 @@ class PackageConfig
     [path_flags, other_flags]
   end
 
-  def normalize_path_flags(path_flags, flag_option)
-    return path_flags unless /-mingw(?:32|-ucrt)\z/ === RUBY_PLATFORM
+  def normalize_path_flag(path_flag, flag_option)
+    return path_flag unless /-mingw(?:32|-ucrt)\z/ === RUBY_PLATFORM
 
     pkg_config_prefix = self.class.native_pkg_config_prefix
-    return path_flags unless pkg_config_prefix
+    return path_flag unless pkg_config_prefix
 
     mingw_dir = pkg_config_prefix.basename.to_s
-    path_flags.collect do |path_flag|
-      path = path_flag.sub(/\A#{Regexp.escape(flag_option)}/, "")
-      path = path.sub(/\A\/#{Regexp.escape(mingw_dir)}/i) do
-        pkg_config_prefix.to_s
-      end
-      "#{flag_option}#{path}"
+    path = path_flag.sub(/\A#{Regexp.escape(flag_option)}/, "")
+    path = path.sub(/\A\/#{Regexp.escape(mingw_dir)}/i) do
+      pkg_config_prefix.to_s
     end
+    "#{flag_option}#{path}"
   end
 
   def normalize_cflags(cflags)
@@ -457,31 +575,32 @@ class PackageConfig
   end
 
   def collect_libs
-    target_packages = sort_packages(required_packages + [self])
+    target_packages = [*required_packages, self]
     libs_set = []
     target_packages.each do |package|
       libs_set << package.declaration("Libs")
     end
-    all_flags = split_lib_flags(libs_set.join(" "))
-    path_flags, other_flags = all_flags.partition {|flag| /\A-L/ =~ flag}
-    path_flags = normalize_path_flags(path_flags, "-L")
-    path_flags = path_flags.reject do |flag|
+    flags = split_lib_flags(libs_set.join(" "))
+    flags = flags.collect do |flag|
+      flag = normalize_path_flag(flag, "-L") if flag.start_with?("-L")
+      flag
+    end
+    flags = flags.reject do |flag|
       /\A-L\/usr\/lib(?:64|x32)?\z/ =~ flag
     end
-    path_flags = path_flags.uniq
+    flags = flags.uniq
     if @msvc_syntax
-      path_flags = path_flags.collect do |flag|
-        flag.gsub(/\A-L/, "/libpath:")
-      end
-      other_flags = other_flags.collect do |flag|
-        if /\A-l/ =~ flag
-          "#{$POSTMATCH}.lib"
+      flags = flags.collect do |flag|
+        if flag.start_with?("-L")
+          flag.gsub(/\A-L/, "/libpath:")
+        elsif flag.start_with?("-l")
+          "#{flag[2..-1]}.lib"
         else
           flag
         end
       end
     end
-    [path_flags, other_flags]
+    flags
   end
 
   def split_lib_flags(libs_command_line)
@@ -514,13 +633,18 @@ class PackageConfig
     @declarations = {}
     File.open(pc_path) do |input|
       input.each_line do |line|
+        if line.dup.force_encoding("UTF-8").valid_encoding?
+          line.force_encoding("UTF-8")
+        end
         line = line.gsub(/#.*/, "").strip
         next if line.empty?
         case line
         when /^(#{IDENTIFIER_RE})\s*=\s*/
-          @variables[$1] = $POSTMATCH.strip
+          match = Regexp.last_match
+          @variables[match[1]] = match.post_match.strip
         when /^(#{IDENTIFIER_RE})\s*:\s*/
-          @declarations[$1] = $POSTMATCH.strip
+          match = Regexp.last_match
+          @declarations[match[1]] = match.post_match.strip
         end
       end
     end
@@ -566,118 +690,3 @@ class PackageConfig
     end
   end
 end
-
-module PKGConfig
-  @@paths = []
-  @@override_variables = {}
-
-  module_function
-  def add_path(path)
-    @@paths << path
-  end
-
-  def set_override_variable(key, value)
-    @@override_variables[key] = value
-  end
-
-  def msvc?
-    /mswin/.match(RUBY_PLATFORM) and /^cl\b/.match(RbConfig::CONFIG["CC"])
-  end
-
-  def package_config(package)
-    PackageConfig.new(package,
-                      :msvc_syntax => msvc?,
-                      :override_variables => @@override_variables,
-                      :paths => @@paths)
-  end
-
-  def exist?(pkg)
-    package_config(pkg).exist?
-  end
-
-  def libs(pkg)
-    package_config(pkg).libs
-  end
-
-  def libs_only_l(pkg)
-    package_config(pkg).libs_only_l
-  end
-
-  def libs_only_L(pkg)
-    package_config(pkg).libs_only_L
-  end
-
-  def cflags(pkg)
-    package_config(pkg).cflags
-  end
-
-  def cflags_only_I(pkg)
-    package_config(pkg).cflags_only_I
-  end
-
-  def cflags_only_other(pkg)
-    package_config(pkg).cflags_only_other
-  end
-
-  def modversion(pkg)
-    package_config(pkg).version
-  end
-
-  def description(pkg)
-    package_config(pkg).description
-  end
-
-  def variable(pkg, name)
-    package_config(pkg).variable(name)
-  end
-
-  def check_version?(pkg, major=0, minor=0, micro=0)
-    return false unless exist?(pkg)
-    ver = modversion(pkg).split(".").collect {|item| item.to_i}
-    (0..2).each {|i| ver[i] = 0 unless ver[i]}
-
-    (ver[0] > major ||
-     (ver[0] == major && ver[1] > minor) ||
-     (ver[0] == major && ver[1] == minor &&
-      ver[2] >= micro))
-  end
-
-  def have_package(pkg, major=nil, minor=0, micro=0)
-    message = "#{pkg}"
-    unless major.nil?
-      message << " version (>= #{major}.#{minor}.#{micro})"
-    end
-    major ||= 0
-    result = checking_for(checking_message(message), "%s") do
-      if check_version?(pkg, major, minor, micro)
-        "yes (#{modversion(pkg)})"
-      else
-        if exist?(pkg)
-          "no (#{modversion(pkg)})"
-        else
-          "no (nonexistent)"
-        end
-      end
-    end
-    enough_version = result.start_with?("yes")
-    if enough_version
-      libraries = libs_only_l(pkg)
-      dldflags = libs(pkg)
-      dldflags = (Shellwords.shellwords(dldflags) -
-                  Shellwords.shellwords(libraries))
-      dldflags = dldflags.map {|s| /\s/ =~ s ? "\"#{s}\"" : s }.join(" ")
-      $libs   += " " + libraries
-      if /mswin/ =~ RUBY_PLATFORM
-        $DLDFLAGS += " " + dldflags
-      else
-        $LDFLAGS += " " + dldflags
-      end
-      $CFLAGS += " " + cflags_only_other(pkg)
-      if defined?($CXXFLAGS)
-        $CXXFLAGS += " " + cflags_only_other(pkg)
-      end
-      $INCFLAGS += " " + cflags_only_I(pkg)
-    end
-    enough_version
-  end
-end
